'use strict';
/*
 * Transforms pages[0..length].content from markdown to HTML
 */

var marked = require('marked');
var renderer = new marked.Renderer();
var highlightjs = require('highlight.js');
var path = require('path');
var jobs = require('./jobs');

var toURL = function(url) {
  return jobs.run('toURL', url);
}

renderer.heading = function(text, level, raw) {
  return '<h'  +
    (level+2)  +
    ' id="'    +
    toURL(raw) +
    '">'       +
    text       +
    '</h'      +
    (level+2)  +
    '>\n';
}

renderer.paragraph = function(text) {
  if(text.match('\\<img')) return '<div class="has-image">'+text+'</div>\n';
  else return '<p>'+text+'</p>\n';
}

var renderImage = function(href, title, text, curFilename) {
  var parseParams = function(params) {
    for(let param of params.split(',')) {
      param = param.trim();
      // 123x or x123 or 123x123
      if(param.match(/[0-9]*x[0-9]+/) || param.match(/[0-9]+x[0-9]*/)) {
        options.size = param.split('x');
        if(options.size !== undefined) {
          if(options.size[0].length > 0) {
            options.width = options.size[0];
          }
          if(options.size.length > 1 && options.size[1].length > 0) {
            options.height = options.size[1];
          }
        }
      } else if(param == 'nolightbox') {
        options.nolightbox = true;
      }
    }
  }

  href = href.split(path.sep).join('/');
  href = href.split(' =');
  var params = href[1];
  var options = {};

  if(params) {
    parseParams(params);
  }
  href = href[0];
  if(href.match('//(www.)?youtube.com/watch') || href.match('//youtu.be/')) {
    var id;
    if(href.match('//youtube.com/watch')) {
      id = href.substr(href.indexOf('v=')+2).split('&')[0];
    } else {
      id = href.substr(href.indexOf('.be/')+4).split('?')[0];
    }

    let out = '<div class="video-container"><div class="videodiv">';
    out += '<iframe src="https://www.youtube.com/embed/'+id+'" frameborder="0" allowfullscreen></iframe>'
    out += '</div></div>'

    return out;
  } else {
    var rel = path.relative(process.cwd(),path.resolve(curFilename,href));

    var out = '<img src="' + href + '" alt="' + text + '"';
    if(options.width) {
      out += ' width="' + options.width + '"';
    }
    if(options.height) {
      out += ' height="' + options.height + '"';
    }
    if (title) {
      out += ' title="' + title + '"';
    }

    out += '>';
    if(!options.nolightbox) { // if lightbox
      var a = '<a href="'+href+'"';
      a += ' data-lightbox="group"';
      a += ' data-title="' + text + '"';
      a += '>';
      out = a+out+'</a>';
    }
    return out;
  }
}

renderer._link = renderer.link;

marked.setOptions({
  renderer: renderer,
  gfm: true,
  tables: true,
  breaks: false,
  pedantic: false,
  sanitize: false,
  smartLists: true,
  smartypants: false,
});

module.exports = function(obj) {
  console.log('Build step: Markdown');
  var options = obj.config.options;
  renderer.link = function(href, title, text) {
    if(options.httponly) {
      if(href.match(/^https:\/\/ok1kvk.cz/))
        href = href.replace('https://', 'http://');
    }
    if(options.httpsonly) {
      if(href.match(/^http:\/\/ok1kvk.cz/))
        href = href.replace('http://', 'https://');
    }

    return renderer._link(href,title,text);
  };

  obj.pages.forEach(function(article) {
    renderer.image = function(href, title, text) {
      return renderImage(href, title, text, article.filename);
    };

    article.content = marked(article.content, {
      highlight: function (code) {
        var h;
        if(highlightjs.getLanguage(article.metadata.lang))
          h = highlightjs.highlight(article.metadata.lang, code, true);
        else
          h = highlightjs.highlightAuto(code);
        return h.value;
      },
      renderer: renderer
    });
  });
  return Promise.resolve(obj);
}
